@@ -614,8 +614,13 @@ static guint
 get_ical_percent_complete (icalcomponent *ical)
 {
   icalproperty *prop;
+  icalproperty_status status;
   int           percent_complete;
 
+  status = icalcomponent_get_status (ical);
+  if (status == ICAL_STATUS_COMPLETED)
+    return 100;
+
   prop = icalcomponent_get_first_property (ical, ICAL_COMPLETED_PROPERTY);
   if (prop)
     return 100;
@@ -1918,11 +1923,12 @@ calendar_client_foreach_appointment_day (CalendarClient  *client,
           if (day_time >= month_begin)
             marked_days [day_from_time_t (day_time)] = TRUE;
       
-          if (appointment->end_time && !appointment->is_all_day)
+          if (appointment->end_time)
             {
               int day_offset;
               int duration = appointment->end_time - appointment->start_time;
-              for (day_offset = 1; day_offset <= duration / 86400; day_offset++)
+	      /* mark the days for the appointment, no need to add an extra one when duration is a multiple of 86400 */
+              for (day_offset = 1; day_offset <= duration / 86400 && duration != day_offset * 86400; day_offset++)
                 {
                   GTime day_time = appointment->start_time + day_offset * 86400;
 
