@@ -42,9 +42,9 @@
 #include "panel-stock-icons.h"
 #include "xstuff.h"
 
-#define PANEL_APPLET_FRAME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET_FRAME, PanelAppletFramePrivate))
+G_DEFINE_TYPE (PanelAppletFrame, panel_applet_frame, GTK_TYPE_EVENT_BOX);
 
-#undef PANEL_APPLET_FRAME_DEBUG
+#define PANEL_APPLET_FRAME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET_FRAME, PanelAppletFramePrivate))
 
 #define HANDLE_SIZE 10
 
@@ -67,7 +67,6 @@ struct _PanelAppletFramePrivate {
 	PanelOrientation                orientation;
 
 	gchar                          *iid;
-	gboolean			moving_focus_out;
 
 	GtkAllocation                   child_allocation;
 	GdkRectangle                    handle_rect;
@@ -83,8 +82,6 @@ typedef struct {
 	char             *id;
 } PanelAppletFrameActivating;
 
-static GObjectClass *parent_class;
-
 /* Keep in sync with panel-applet.h. Uggh.
  */	
 typedef enum {
@@ -94,6 +91,8 @@ typedef enum {
 	APPLET_HAS_HANDLE   = 1 << 2
 } PanelAppletFlags;
 
+static GSList *no_reload_applets = NULL;
+
 static void panel_applet_frame_cnx_broken (ORBitConnection  *cnx,
 					   PanelAppletFrame *frame);
 
@@ -183,6 +182,7 @@ panel_applet_frame_set_flags_from_any (PanelAppletFrame *frame,
 {
 	gboolean major;
 	gboolean minor;
+	gboolean old_has_handle;
 	int      flags;
 	
 	flags = BONOBO_ARG_GET_SHORT (any);
@@ -193,7 +193,17 @@ panel_applet_frame_set_flags_from_any (PanelAppletFrame *frame,
 	panel_widget_set_applet_expandable (
 		frame->priv->panel, GTK_WIDGET (frame), major, minor);
 
+	old_has_handle = frame->priv->has_handle;
 	frame->priv->has_handle = (flags & APPLET_HAS_HANDLE) != 0;
+
+	if (!old_has_handle && frame->priv->has_handle) {
+		/* we've added an handle, so we need to get the background for
+		 * it */
+		PanelBackground *background;
+
+		background = &frame->priv->panel->background;
+		panel_applet_frame_change_background (frame, background->type);
+	}
 }
 
 static void
@@ -332,6 +342,10 @@ panel_applet_frame_load (const gchar *iid,
 	g_return_if_fail (panel != NULL);
 	g_return_if_fail (id != NULL);
 
+	if (g_slist_find_custom (no_reload_applets, id,
+				 (GCompareFunc) strcmp))
+		return;
+
 	if (panel_lockdown_is_applet_disabled (iid))
 		return;
 
@@ -373,7 +387,7 @@ panel_applet_frame_load_from_gconf (PanelWidget *panel_widget,
 	applet_iid = gconf_client_get_string (panel_gconf_get_client (),
 					      key, NULL);
 
-	if (!applet_iid)
+	if (!applet_iid || !applet_iid[0])
 		return;
 
 	panel_applet_frame_load (applet_iid, panel_widget,
@@ -531,6 +545,15 @@ panel_applet_frame_finalize (GObject *object)
 	panel_lockdown_notify_remove (G_CALLBACK (panel_applet_frame_sync_menu_state),
 				      frame);
 
+	if (frame->priv->control) {
+		/* do this before unref'ing every bonobo stuff since it looks
+		 * like we can receive some events when unref'ing them */
+		ORBit_small_unlisten_for_broken (frame->priv->control,
+						 G_CALLBACK (panel_applet_frame_cnx_broken));
+		CORBA_Object_release (frame->priv->control, NULL);
+		frame->priv->control = CORBA_OBJECT_NIL;
+	}
+
 	if (frame->priv->property_bag)
 		bonobo_object_release_unref (
 			frame->priv->property_bag, NULL);
@@ -543,17 +566,10 @@ panel_applet_frame_finalize (GObject *object)
 		bonobo_object_unref (
 			BONOBO_OBJECT (frame->priv->ui_component));
 
-	if (frame->priv->control) {
-		ORBit_small_unlisten_for_broken (frame->priv->control,
-						 G_CALLBACK (panel_applet_frame_cnx_broken));
-		CORBA_Object_release (frame->priv->control, NULL);
-		frame->priv->control = CORBA_OBJECT_NIL;
-	}
-
 	g_free (frame->priv->iid);
 	frame->priv->iid = NULL;
 
-        parent_class->finalize (object);
+        G_OBJECT_CLASS (panel_applet_frame_parent_class)->finalize (object);
 }
 
 static void
@@ -602,7 +618,7 @@ panel_applet_frame_expose (GtkWidget      *widget,
 			   GdkEventExpose *event)
 {
 	if (GTK_WIDGET_DRAWABLE (widget)) {
-		GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+		GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->expose_event (widget, event);
 
 		panel_applet_frame_paint (widget, &event->area);
 
@@ -645,7 +661,7 @@ panel_applet_frame_size_request (GtkWidget      *widget,
 	bin = GTK_BIN (widget);
 
 	if (!frame->priv->has_handle) {
-		GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
+		GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->size_request (widget, requisition);
 		return;
 	}
   
@@ -695,7 +711,7 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 	bin = GTK_BIN (widget);
 
 	if (!frame->priv->has_handle) {
-		GTK_WIDGET_CLASS (parent_class)->size_allocate (widget,
+		GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->size_allocate (widget,
 								allocation);
 		panel_applet_frame_update_background_size (frame,
 							   &old_allocation,
@@ -999,8 +1015,19 @@ panel_applet_frame_loading_failed_response (GtkWidget *dialog,
 
 	if (response == LOADING_FAILED_RESPONSE_DELETE &&
 	    !panel_lockdown_get_locked_down () &&
-	    panel_profile_id_lists_are_writable ())
+	    panel_profile_id_lists_are_writable ()) {
+		GSList *item;
+
+		item = g_slist_find_custom (no_reload_applets, id,
+					    (GCompareFunc) strcmp);
+		if (item) {
+			g_free (item->data);
+			no_reload_applets = g_slist_delete_link (no_reload_applets,
+								 item);
+		}
+
 		panel_profile_remove_from_list (PANEL_GCONF_APPLETS, id);
+	}
 
 	g_free (id);
 }
@@ -1013,6 +1040,9 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 	char      *problem_txt;
 	gboolean   locked_down;
 
+	no_reload_applets = g_slist_prepend (no_reload_applets,
+					     g_strdup (id));
+
 	locked_down = panel_lockdown_get_locked_down ();
 
 	problem_txt = g_strdup_printf (_("The panel encountered a problem "
@@ -1057,14 +1087,11 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 }
 
 static void
-panel_applet_frame_class_init (PanelAppletFrameClass *klass,
-			       gpointer               dummy)
+panel_applet_frame_class_init (PanelAppletFrameClass *klass)
 {
 	GObjectClass   *gobject_class = (GObjectClass *) klass;
 	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	gobject_class->finalize = panel_applet_frame_finalize;
 
 	widget_class->expose_event         = panel_applet_frame_expose;
@@ -1077,8 +1104,7 @@ panel_applet_frame_class_init (PanelAppletFrameClass *klass,
 }
 
 static void
-panel_applet_frame_instance_init (PanelAppletFrame      *frame,
-				  PanelAppletFrameClass *klass)
+panel_applet_frame_init (PanelAppletFrame *frame)
 {
 	frame->priv = PANEL_APPLET_FRAME_GET_PRIVATE (frame);
 
@@ -1088,37 +1114,9 @@ panel_applet_frame_instance_init (PanelAppletFrame      *frame,
 	frame->priv->panel            = NULL;
 	frame->priv->orientation      = PANEL_ORIENTATION_TOP;
 	frame->priv->applet_info      = NULL;
-	frame->priv->moving_focus_out = FALSE;
 	frame->priv->has_handle       = FALSE;
 }
 
-GType
-panel_applet_frame_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (PanelAppletFrameClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) panel_applet_frame_class_init,
-			NULL,
-			NULL,
-			sizeof (PanelAppletFrame),
-			0,
-			(GInstanceInitFunc) panel_applet_frame_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (GTK_TYPE_EVENT_BOX,
-					       "PanelAppletFrame",
-					       &info, 0);
-	}
-
-	return type;
-}
-
 static GNOME_Vertigo_PanelAppletShell
 panel_applet_frame_get_applet_shell (Bonobo_Control control)
 {
